# Generated by Django 4.2.17 on 2026-01-21 22:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("brevo_analytics", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="brevomessage",
            options={
                "ordering": ["-sent_at", "-sent_date", "subject"],
                "verbose_name": "Brevo Message",
                "verbose_name_plural": "Brevo Messages",
            },
        ),
        migrations.AddField(
            model_name="brevomessage",
            name="sent_at",
            field=models.DateTimeField(
                blank=True,
                db_index=True,
                help_text="Timestamp del primo messaggio inviato",
                null=True,
            ),
        ),
        migrations.AddIndex(
            model_name="brevomessage",
            index=models.Index(
                fields=["-sent_at"], name="brevo_messa_sent_at_1c1f09_idx"
            ),
        ),
    ]



from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from django.db.models import Sum, Avg, Count, Q
from django.conf import settings
import requests
from urllib.parse import quote
from .models import BrevoMessage, BrevoEmail
from .serializers import (
    BrevoMessageSerializer,
    BrevoEmailListSerializer,
    BrevoEmailDetailSerializer,
    MessageBrevoEmailsSerializer,
    GlobalBrevoEmailsSerializer
)


@api_view(['GET'])
@permission_classes([IsAdminUser])
def dashboard_api(request):
    """
    Dashboard KPI + ultimi 20 messaggi.

    GET /api/dashboard/
    """
    # Calcola KPI globali
    all_messages = BrevoMessage.objects.all()

    kpi = all_messages.aggregate(
        total_sent=Sum('total_sent'),
        total_delivered=Sum('total_delivered'),
        total_opened=Sum('total_opened'),
        total_clicked=Sum('total_clicked'),
        total_bounced=Sum('total_bounced'),
        total_blocked=Sum('total_blocked'),
    )

    # Calcola rates globali
    total_sent = kpi['total_sent'] or 0
    total_delivered = kpi['total_delivered'] or 0

    if total_sent > 0:
        delivery_rate = round(total_delivered / total_sent * 100, 2)
    else:
        delivery_rate = 0.0

    if total_delivered > 0:
        open_rate = round((kpi['total_opened'] or 0) / total_delivered * 100, 2)
        click_rate = round((kpi['total_clicked'] or 0) / total_delivered * 100, 2)
    else:
        open_rate = 0.0
        click_rate = 0.0

    kpi_data = {
        'total_sent': total_sent,
        'delivery_rate': delivery_rate,
        'open_rate': open_rate,
        'click_rate': click_rate,
        'total_bounced': kpi['total_bounced'] or 0,
        'total_blocked': kpi['total_blocked'] or 0,
    }

    # Ultimi 20 messaggi
    recent_messages = BrevoMessage.objects.all()[:20]
    messages_data = BrevoMessageSerializer(recent_messages, many=True).data

    return Response({
        'kpi': kpi_data,
        'recent_messages': messages_data
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def messages_list_api(request):
    """
    Lista di tutti i messaggi (per "Mostra tutti").

    GET /api/messages/
    """
    messages = BrevoMessage.objects.all()
    serializer = BrevoMessageSerializer(messages, many=True)

    return Response({
        'messages': serializer.data
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def message_emails_api(request, message_id):
    """
    BrevoEmail per messaggio specifico.

    GET /api/messages/:id/emails/
    """
    try:
        message = BrevoMessage.objects.get(id=message_id)
    except BrevoMessage.DoesNotExist:
        return Response({'error': 'Message not found'}, status=404)

    emails = message.emails.all()

    # Filtro per status se presente query param
    status_filter = request.GET.get('status')
    if status_filter:
        emails = emails.filter(current_status=status_filter)

    message_data = BrevoMessageSerializer(message).data
    emails_data = BrevoEmailListSerializer(emails, many=True).data

    return Response({
        'message': message_data,
        'emails': emails_data
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def emails_bounced_api(request):
    """
    Tutte le email bounced (cross-message).

    GET /api/emails/bounced/
    """
    emails = BrevoEmail.objects.filter(current_status='bounced').select_related('message')
    serializer = GlobalBrevoEmailsSerializer(emails, many=True)

    return Response({
        'emails': serializer.data
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def emails_blocked_api(request):
    """
    Tutte le email blocked (cross-message).

    GET /api/emails/blocked/
    """
    emails = BrevoEmail.objects.filter(current_status='blocked').select_related('message')
    serializer = GlobalBrevoEmailsSerializer(emails, many=True)

    return Response({
        'emails': serializer.data
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def email_detail_api(request, email_id):
    """
    Dettaglio email singola (per modale).

    GET /api/emails/:id/

    Se l'email ha status 'blocked' e non ha blacklist_info cached,
    interroga l'API Brevo per ottenerle.
    """
    try:
        email = BrevoEmail.objects.select_related('message').get(id=email_id)
    except BrevoEmail.DoesNotExist:
        return Response({'error': 'BrevoEmail not found'}, status=404)

    # Se è blocked e non abbiamo blacklist_info, prova a ottenerla
    if email.current_status == 'blocked' and not email.blacklist_info:
        brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
        api_key = brevo_config.get('API_KEY', '')

        if api_key:
            try:
                headers = {
                    'api-key': api_key,
                    'accept': 'application/json'
                }

                response = requests.get(
                    f"https://api.brevo.com/v3/smtp/blockedContacts",
                    headers=headers,
                    params={
                        'email': email.recipient_email,
                        'limit': 50
                    },
                    timeout=5
                )

                if response.status_code == 200:
                    data = response.json()
                    contacts = data.get('contacts', [])

                    # Find matching contact
                    for contact in contacts:
                        if contact.get('email', '').lower() == email.recipient_email.lower():
                            # Verify this contact belongs to our client:
                            # - If senderEmail matches ALLOWED_SENDERS: it's ours
                            # - If senderEmail is empty/None: check if in local DB (email object already loaded)
                            allowed_senders = brevo_config.get('ALLOWED_SENDERS', [])
                            if allowed_senders:
                                if isinstance(allowed_senders, str):
                                    allowed_senders = [allowed_senders]

                                contact_senders = contact.get('senderEmail', [])
                                if isinstance(contact_senders, str):
                                    contact_senders = [contact_senders]
                                elif not isinstance(contact_senders, list):
                                    contact_senders = []

                                # Check if any of the contact's senders match our allowed list
                                has_our_sender = any(sender in allowed_senders for sender in contact_senders)

                                # If no sender, we already have the email in DB (we're viewing its detail)
                                # So it's definitely ours
                                if not has_our_sender and len(contact_senders) > 0:
                                    # Has senders but not ours - skip
                                    break

                            # Normalize data
                            reason = contact.get('reason', 'unknown')
                            if isinstance(reason, dict):
                                # Extract code first (will be translated in frontend), fallback to message
                                reason = reason.get('code') or reason.get('message') or str(reason)
                            elif not isinstance(reason, str):
                                reason = str(reason) if reason else 'unknown'

                            senders = contact.get('senderEmail', [])
                            if not isinstance(senders, list):
                                if isinstance(senders, str):
                                    senders = [senders]
                                else:
                                    senders = []

                            # Cache blacklist info
                            from django.utils import timezone
                            email.blacklist_info = {
                                'reason': reason,
                                'blocked_at': contact.get('blockedAt', ''),
                                'senders': senders,
                                'checked_at': timezone.now().isoformat()
                            }
                            email.save(update_fields=['blacklist_info', 'updated_at'])
                            break

            except Exception:
                # Silently fail - not critical for display
                pass

    serializer = BrevoEmailDetailSerializer(email)
    return Response(serializer.data)


@api_view(['GET'])
@permission_classes([IsAdminUser])
def check_blacklist_status_api(request, email_address):
    """
    Verifica se un'email è in blacklist Brevo.

    GET /api/blacklist/:email_address/

    Returns:
        {
            "is_blacklisted": true/false,
            "reason": "hard_bounce" | "complaint" | "unsubscribe" | "manual_block",
            "blocked_at": "2026-01-20T10:00:00Z",
            "senders": ["sender1@example.com", "sender2@example.com"]
        }
    """
    brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
    api_key = brevo_config.get('API_KEY', '')

    if not api_key:
        return Response({
            'error': 'Brevo API key not configured'
        }, status=500)

    headers = {
        'api-key': api_key,
        'accept': 'application/json'
    }

    try:
        # Get blocklist info
        # https://developers.brevo.com/reference/get-transac-blocked-contacts
        response = requests.get(
            f"https://api.brevo.com/v3/smtp/blockedContacts",
            headers=headers,
            params={
                'email': email_address,
                'limit': 50
            },
            timeout=10
        )

        if response.status_code == 401:
            return Response({
                'error': 'Invalid API key'
            }, status=500)

        if response.status_code != 200:
            return Response({
                'error': f'Brevo API error: {response.status_code}',
                'details': response.text[:200]
            }, status=500)

        data = response.json()
        contacts = data.get('contacts', [])

        # Find matching contact
        matching_contact = None
        for contact in contacts:
            if contact.get('email', '').lower() == email_address.lower():
                matching_contact = contact
                break

        if not matching_contact:
            return Response({
                'is_blacklisted': False,
                'email': email_address
            })

        # Verify this contact belongs to our client:
        # - If senderEmail matches ALLOWED_SENDERS: it's ours
        # - If senderEmail is empty/None: check if in local DB
        allowed_senders = brevo_config.get('ALLOWED_SENDERS', [])
        if allowed_senders:
            if isinstance(allowed_senders, str):
                allowed_senders = [allowed_senders]

            contact_senders = matching_contact.get('senderEmail', [])
            if isinstance(contact_senders, str):
                contact_senders = [contact_senders]
            elif not isinstance(contact_senders, list):
                contact_senders = []

            # Check if any of the contact's senders match our allowed list
            has_our_sender = any(sender in allowed_senders for sender in contact_senders)

            # If no sender, check if email is in our local database
            if not has_our_sender and len(contact_senders) == 0:
                has_our_sender = BrevoEmail.objects.filter(
                    recipient_email__iexact=email_address
                ).exists()

            if not has_our_sender:
                # This contact is blocked but not for our client
                return Response({
                    'is_blacklisted': False,
                    'email': email_address
                })

        # Normalize data
        reason = matching_contact.get('reason', 'unknown')
        if isinstance(reason, dict):
            # Extract code first (will be translated in frontend), fallback to message
            reason = reason.get('code') or reason.get('message') or str(reason)
        elif not isinstance(reason, str):
            reason = str(reason) if reason else 'unknown'

        senders = matching_contact.get('senderEmail', [])
        if not isinstance(senders, list):
            if isinstance(senders, str):
                senders = [senders]
            else:
                senders = []

        # Parse blocklist info
        return Response({
            'is_blacklisted': True,
            'email': email_address,
            'reason': reason,
            'blocked_at': matching_contact.get('blockedAt', ''),
            'senders': senders
        })

    except requests.exceptions.Timeout:
        return Response({
            'error': 'Timeout connecting to Brevo API'
        }, status=504)
    except Exception as e:
        return Response({
            'error': f'Unexpected error: {str(e)}'
        }, status=500)


@api_view(['DELETE'])
@permission_classes([IsAdminUser])
def remove_from_blacklist_api(request, email_address):
    """
    Rimuove un'email dalla blacklist Brevo.

    DELETE /api/blacklist/:email_address/

    Returns:
        {"success": true, "message": "Email removed from blacklist"}
    """
    brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
    api_key = brevo_config.get('API_KEY', '')

    if not api_key:
        return Response({
            'error': 'Brevo API key not configured'
        }, status=500)

    headers = {
        'api-key': api_key,
        'accept': 'application/json'
    }

    try:
        # URL-encode the email address
        encoded_email = quote(email_address, safe='')

        # Remove from blocklist
        # https://developers.brevo.com/reference/delete_smtp-blockedcontacts-email
        response = requests.delete(
            f"https://api.brevo.com/v3/smtp/blockedContacts/{encoded_email}",
            headers=headers,
            timeout=10
        )

        if response.status_code == 401:
            return Response({
                'error': 'Invalid API key'
            }, status=500)

        if response.status_code == 404:
            return Response({
                'error': 'Email not found in blacklist'
            }, status=404)

        if response.status_code not in [200, 204]:
            return Response({
                'error': f'Brevo API error: {response.status_code}',
                'details': response.text[:200]
            }, status=500)

        # Clear cached blacklist_info in DB
        BrevoEmail.objects.filter(
            recipient_email__iexact=email_address
        ).update(blacklist_info=None)

        return Response({
            'success': True,
            'message': f'Email {email_address} removed from blacklist'
        })

    except requests.exceptions.Timeout:
        return Response({
            'error': 'Timeout connecting to Brevo API'
        }, status=504)
    except Exception as e:
        return Response({
            'error': f'Unexpected error: {str(e)}'
        }, status=500)


@api_view(['GET'])
@permission_classes([IsAdminUser])
def list_blacklist_api(request):
    """
    Lista completa blacklist Brevo.

    GET /api/blacklist/

    Query params:
        - reason: filter by reason (optional)
        - limit: max results (default 100, max 500)

    Returns:
        {
            "contacts": [...],
            "total": 123,
            "by_reason": {"hard_bounce": 50, "complaint": 10, ...}
        }
    """
    brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
    api_key = brevo_config.get('API_KEY', '')

    if not api_key:
        return Response({
            'error': 'Brevo API key not configured'
        }, status=500)

    headers = {
        'api-key': api_key,
        'accept': 'application/json'
    }

    reason_filter = request.GET.get('reason')
    max_results = min(int(request.GET.get('limit', 100)), 500)

    try:
        all_contacts = []
        offset = 0
        limit = 50

        # Paginate through API
        while len(all_contacts) < max_results:
            response = requests.get(
                "https://api.brevo.com/v3/smtp/blockedContacts",
                headers=headers,
                params={
                    'limit': limit,
                    'offset': offset
                },
                timeout=10
            )

            if response.status_code == 401:
                return Response({
                    'error': 'Invalid API key'
                }, status=500)

            if response.status_code != 200:
                return Response({
                    'error': f'Brevo API error: {response.status_code}',
                    'details': response.text[:200]
                }, status=500)

            data = response.json()
            contacts = data.get('contacts', [])

            if not contacts:
                break

            # Normalize contact data
            for contact in contacts:
                # Ensure reason is a string
                if 'reason' in contact:
                    reason = contact['reason']
                    if isinstance(reason, dict):
                        # Extract code first (will be translated in frontend), fallback to message
                        contact['reason'] = reason.get('code') or reason.get('message') or str(reason)
                    elif not isinstance(reason, str):
                        contact['reason'] = str(reason) if reason else 'unknown'

                # Ensure senderEmail is always a list
                if 'senderEmail' in contact:
                    if not isinstance(contact['senderEmail'], list):
                        if isinstance(contact['senderEmail'], str):
                            contact['senderEmail'] = [contact['senderEmail']]
                        else:
                            contact['senderEmail'] = []

            all_contacts.extend(contacts)
            offset += limit

            if len(contacts) < limit:
                break

        # Filter by allowed senders OR local database:
        # - If senderEmail matches ALLOWED_SENDERS: include (even if old, not in DB)
        # - If senderEmail is empty/None: include only if in local DB (to exclude other clients)
        allowed_senders = brevo_config.get('ALLOWED_SENDERS', [])
        if allowed_senders:
            if isinstance(allowed_senders, str):
                allowed_senders = [allowed_senders]

            # Get all local emails for filtering contacts without sender
            local_emails = set(
                BrevoEmail.objects.values_list('recipient_email', flat=True)
            )
            local_emails_lower = {e.lower() for e in local_emails}

            filtered_contacts = []
            for contact in all_contacts:
                sender_emails = contact.get('senderEmail', [])
                contact_email = contact.get('email', '').lower()

                # senderEmail is always a list now (normalized above)
                if isinstance(sender_emails, list):
                    # Has sender from our allowed list: always include
                    if any(sender in allowed_senders for sender in sender_emails):
                        filtered_contacts.append(contact)
                    # No sender (or empty list): include only if in local DB
                    elif len(sender_emails) == 0 and contact_email in local_emails_lower:
                        filtered_contacts.append(contact)
                elif sender_emails in allowed_senders:
                    # Fallback for non-list (shouldn't happen after normalization)
                    filtered_contacts.append(contact)

            all_contacts = filtered_contacts

        # Filter by reason if requested
        if reason_filter:
            filtered = []
            for c in all_contacts:
                contact_reason = c.get('reason', '')
                # Ensure we're comparing strings
                if isinstance(contact_reason, str) and contact_reason == reason_filter:
                    filtered.append(c)
            all_contacts = filtered

        # Limit results
        all_contacts = all_contacts[:max_results]

        # Sort by blocked date (most recent first)
        all_contacts.sort(key=lambda c: c.get('blockedAt', ''), reverse=True)

        # Check which emails have local data (for bounce details)
        # Note: Some old contacts with senderEmail may not be in local DB
        local_emails = set(
            BrevoEmail.objects.filter(
                recipient_email__in=[c.get('email') for c in all_contacts]
            ).values_list('recipient_email', flat=True)
        )

        for contact in all_contacts:
            contact['has_local_data'] = contact.get('email') in local_emails

        # Group by reason for stats
        by_reason = {}
        for contact in all_contacts:
            reason = contact.get('reason', 'unknown')
            # Ensure reason is a string (API might return unexpected types)
            if not isinstance(reason, str):
                reason = str(reason) if reason else 'unknown'
            by_reason[reason] = by_reason.get(reason, 0) + 1

        return Response({
            'contacts': all_contacts,
            'total': len(all_contacts),
            'by_reason': by_reason
        })

    except requests.exceptions.Timeout:
        return Response({
            'error': 'Timeout connecting to Brevo API'
        }, status=504)
    except Exception as e:
        return Response({
            'error': f'Unexpected error: {str(e)}'
        }, status=500)


@api_view(['POST'])
@permission_classes([IsAdminUser])
def enrich_blocked_emails_api(request):
    """
    Arricchisce email blocked nel DB con info da Brevo API.

    POST /api/blacklist/enrich/

    Body:
        {
            "force": false  // Re-enrich anche quelle già processate
        }

    Returns:
        {
            "enriched": 123,
            "not_found": 5,
            "total_processed": 128
        }
    """
    brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
    api_key = brevo_config.get('API_KEY', '')

    if not api_key:
        return Response({
            'error': 'Brevo API key not configured'
        }, status=500)

    force = request.data.get('force', False)

    # Get blocked emails to process
    if force:
        blocked_emails = BrevoEmail.objects.filter(current_status='blocked')
    else:
        blocked_emails = BrevoEmail.objects.filter(
            current_status='blocked',
            blacklist_info__isnull=True
        )

    total = blocked_emails.count()

    if total == 0:
        return Response({
            'enriched': 0,
            'not_found': 0,
            'total_processed': 0,
            'message': 'No blocked emails to process'
        })

    headers = {
        'api-key': api_key,
        'accept': 'application/json'
    }

    enriched = 0
    not_found = 0

    from django.utils import timezone
    import time

    for email_obj in blocked_emails:
        email_address = email_obj.recipient_email

        try:
            response = requests.get(
                "https://api.brevo.com/v3/smtp/blockedContacts",
                headers=headers,
                params={
                    'email': email_address,
                    'limit': 50
                },
                timeout=5
            )

            if response.status_code == 429:
                # Rate limited, wait and retry
                time.sleep(5)
                continue

            if response.status_code == 200:
                data = response.json()
                contacts = data.get('contacts', [])

                # Find matching contact
                matching = None
                for contact in contacts:
                    if contact.get('email', '').lower() == email_address.lower():
                        matching = contact
                        break

                if matching:
                    # Verify this contact belongs to our client:
                    # - If senderEmail matches ALLOWED_SENDERS: it's ours
                    # - If senderEmail is empty/None: it's ours (we're enriching local DB emails)
                    allowed_senders = brevo_config.get('ALLOWED_SENDERS', [])
                    if allowed_senders:
                        if isinstance(allowed_senders, str):
                            allowed_senders = [allowed_senders]

                        contact_senders = matching.get('senderEmail', [])
                        if isinstance(contact_senders, str):
                            contact_senders = [contact_senders]
                        elif not isinstance(contact_senders, list):
                            contact_senders = []

                        # Check if any of the contact's senders match our allowed list
                        has_our_sender = any(sender in allowed_senders for sender in contact_senders)

                        # If no sender, it's ours (we're processing emails from our local DB)
                        # If has senders but not ours, skip
                        if len(contact_senders) > 0 and not has_our_sender:
                            # This contact is blocked but not for our client - skip
                            not_found += 1
                            continue

                    # Normalize data
                    reason = matching.get('reason', 'unknown')
                    if isinstance(reason, dict):
                        # Extract code first (will be translated in frontend), fallback to message
                        reason = reason.get('code') or reason.get('message') or str(reason)
                    elif not isinstance(reason, str):
                        reason = str(reason) if reason else 'unknown'

                    senders = matching.get('senderEmail', [])
                    if not isinstance(senders, list):
                        if isinstance(senders, str):
                            senders = [senders]
                        else:
                            senders = []

                    email_obj.blacklist_info = {
                        'reason': reason,
                        'blocked_at': matching.get('blockedAt', ''),
                        'senders': senders,
                        'checked_at': timezone.now().isoformat()
                    }
                    email_obj.save(update_fields=['blacklist_info', 'updated_at'])
                    enriched += 1
                else:
                    not_found += 1

        except Exception:
            continue

    return Response({
        'enriched': enriched,
        'not_found': not_found,
        'total_processed': total
    })


@api_view(['GET'])
@permission_classes([IsAdminUser])
def get_bounce_details_api(request, email_address):
    """
    Recupera i dettagli del bounce per un indirizzo email.

    Prima controlla il database locale, poi interroga Brevo API se necessario.

    GET /api/bounce-details/<email_address>/

    Returns:
        - email: indirizzo email
        - bounce_reason: motivo del bounce
        - bounce_type: hard/soft
        - timestamp: data/ora del bounce
        - subject: oggetto del messaggio (se da DB locale)
        - source: 'local' o 'api'
    """
    # 1. Cerca prima nel database locale
    emails = BrevoEmail.objects.filter(
        recipient_email=email_address,
        current_status='bounced'
    ).order_by('-sent_at')

    if emails.exists():
        email = emails.first()

        # Trova l'evento bounce più recente nell'array events
        bounce_event = None
        if email.events:
            for event in reversed(email.events):
                if event.get('type') == 'bounced':
                    bounce_event = event
                    break

        if bounce_event:
            return Response({
                'email': email_address,
                'bounce_reason': bounce_event.get('bounce_reason', 'Motivo non specificato'),
                'bounce_type': bounce_event.get('bounce_type', 'unknown'),
                'timestamp': bounce_event.get('timestamp', ''),
                'subject': email.message.subject if email.message else 'N/A',
                'sent_at': email.sent_at.isoformat() if email.sent_at else None,
                'source': 'local'
            })

    # 2. Se non trovato localmente, interroga Brevo API
    brevo_config = getattr(settings, 'BREVO_ANALYTICS', {})
    api_key = brevo_config.get('API_KEY', '')

    if not api_key:
        return Response({
            'error': 'Dettagli non disponibili',
            'detail': 'Email non presente nel database locale e API key non configurata'
        }, status=404)

    headers = {
        'api-key': api_key,
        'accept': 'application/json'
    }

    try:
        # Query Brevo API per eventi bounce di questa email
        # Range ampio per trovare bounce anche vecchi (ultimi 3 anni)
        from datetime import datetime, timedelta
        end_date = datetime.now().strftime('%Y-%m-%d')
        start_date = (datetime.now() - timedelta(days=1095)).strftime('%Y-%m-%d')  # 3 anni

        # Prova prima con hard_bounce
        for event_type in ['hard_bounce', 'soft_bounce']:
            params = {
                'event': event_type,
                'email': email_address,
                'startDate': start_date,
                'endDate': end_date,
                'limit': 1,
                'sort': 'desc'
            }

            response = requests.get(
                "https://api.brevo.com/v3/smtp/statistics/events",
                headers=headers,
                params=params,
                timeout=10
            )

            if response.status_code == 200:
                data = response.json()
                events = data.get('events', [])

                if events:
                    event = events[0]
                    return Response({
                        'email': email_address,
                        'bounce_reason': event.get('reason') or event.get('error') or 'Motivo non specificato',
                        'bounce_type': 'hard' if event_type == 'hard_bounce' else 'soft',
                        'timestamp': event.get('date', ''),
                        'subject': event.get('subject', 'N/A'),
                        'source': 'api'
                    })

        # Nessun evento trovato
        return Response({
            'error': 'Nessun evento bounce trovato',
            'detail': 'Nessun bounce negli ultimi 3 anni per questo indirizzo nelle statistiche Brevo'
        }, status=404)

    except requests.exceptions.Timeout:
        return Response({
            'error': 'Timeout connecting to Brevo API'
        }, status=504)
    except Exception as e:
        return Response({
            'error': f'Errore durante la richiesta a Brevo: {str(e)}'
        }, status=500)



from django.test import TestCase
from django.utils import timezone
from datetime import datetime
from .models import BrevoMessage, Email

class BrevoModelsTestCase(TestCase):
    def setUp(self):
        self.message = BrevoMessage.objects.create(
            subject="Test Email",
            sent_date=timezone.now().date()
        )

    def test_email_creation(self):
        email = Email.objects.create(
            message=self.message,
            brevo_message_id="<test123@example.com>",
            recipient_email="test@example.com",
            sent_at=timezone.now()
        )
        self.assertEqual(email.current_status, 'sent')

    def test_add_event(self):
        email = Email.objects.create(
            message=self.message,
            brevo_message_id="<test456@example.com>",
            recipient_email="test2@example.com",
            sent_at=timezone.now()
        )

        email.add_event('delivered', timezone.now())
        self.assertEqual(email.current_status, 'delivered')
        self.assertEqual(len(email.events), 1)

    def test_status_hierarchy(self):
        email = Email.objects.create(
            message=self.message,
            brevo_message_id="<test789@example.com>",
            recipient_email="test3@example.com",
            sent_at=timezone.now()
        )

        email.add_event('delivered', timezone.now())
        email.add_event('opened', timezone.now())
        email.add_event('clicked', timezone.now())

        self.assertEqual(email.current_status, 'clicked')

    def test_message_stats_update(self):
        # Create multiple emails for the message
        for i in range(5):
            email = Email.objects.create(
                message=self.message,
                brevo_message_id=f"<test{i}@example.com>",
                recipient_email=f"test{i}@example.com",
                sent_at=timezone.now()
            )
            if i < 3:
                email.add_event('delivered', timezone.now())
            if i < 2:
                email.add_event('opened', timezone.now())

        # Manually update stats after all emails created
        self.message.update_stats()

        # Refresh message from DB
        self.message.refresh_from_db()

        self.assertEqual(self.message.total_sent, 5)
        self.assertEqual(self.message.total_delivered, 3)
        self.assertEqual(self.message.total_opened, 2)
        self.assertEqual(self.message.delivery_rate, 60.0)

    def test_duplicate_event_prevention(self):
        email = Email.objects.create(
            message=self.message,
            brevo_message_id="<testdup@example.com>",
            recipient_email="testdup@example.com",
            sent_at=timezone.now()
        )

        timestamp = timezone.now()
        added1 = email.add_event('delivered', timestamp)
        added2 = email.add_event('delivered', timestamp)

        self.assertTrue(added1)
        self.assertFalse(added2)  # Should not add duplicate
        self.assertEqual(len(email.events), 1)



# Generated by Django 4.2.17 on 2026-01-22 00:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("brevo_analytics", "0003_rename_email_brevoemail_and_unique_constraint"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="brevoemail",
            options={
                "ordering": ["-sent_at"],
                "verbose_name": "Brevo Email",
                "verbose_name_plural": "Brevo Emails",
            },
        ),
        migrations.AddField(
            model_name="brevoemail",
            name="blacklist_info",
            field=models.JSONField(
                blank=True,
                help_text="Cached blacklist info from Brevo API: {reason, blocked_at, senders, checked_at}",
                null=True,
            ),
        ),
    ]



from django.core.management.base import BaseCommand
from django.utils import timezone
from datetime import datetime
import csv
import uuid
from brevo_analytics.models import BrevoMessage, BrevoEmail
import pytz

class Command(BaseCommand):
    help = 'Import historical Brevo data from CSV files'

    def add_arguments(self, parser):
        parser.add_argument('emails_csv', type=str, help='Path to emails_import.csv')
        parser.add_argument('events_csv', type=str, help='Path to email_events_import.csv')

    def handle(self, *args, **options):
        # Import emails
        self.stdout.write("Importing emails...")
        emails_created = 0
        email_id_map = {}  # CSV id -> Django UUID

        with open(options['emails_csv'], 'r', encoding='utf-8') as f:
            reader = csv.DictReader(f)
            for row in reader:
                csv_id = row['id']
                brevo_message_id = row['brevo_email_id']
                subject = row['subject']
                sent_at_str = row['sent_at']

                # Parse datetime and make it timezone-aware
                sent_at = datetime.fromisoformat(sent_at_str)
                if sent_at.tzinfo is None:
                    # If naive, assume UTC
                    sent_at = pytz.UTC.localize(sent_at)

                sent_date = sent_at.date()

                # Get or create message
                message, _ = BrevoMessage.objects.get_or_create(
                    subject=subject,
                    sent_date=sent_date
                )

                # Create email
                email, created = BrevoEmail.objects.get_or_create(
                    brevo_message_id=brevo_message_id,
                    defaults={
                        'message': message,
                        'recipient_email': row['recipient_email'],
                        'sent_at': sent_at,
                        'events': []
                    }
                )

                if created:
                    emails_created += 1
                    email_id_map[csv_id] = str(email.id)

        self.stdout.write(f"✓ Imported {emails_created} emails")

        # Import events
        self.stdout.write("Importing events...")
        events_created = 0

        with open(options['events_csv'], 'r', encoding='utf-8') as f:
            reader = csv.DictReader(f)
            for row in reader:
                email_csv_id = row['email_id']

                if email_csv_id not in email_id_map:
                    continue

                email_uuid = email_id_map[email_csv_id]

                try:
                    email = BrevoEmail.objects.get(id=email_uuid)
                except BrevoEmail.DoesNotExist:
                    continue

                event_type = row['event_type']
                event_timestamp_str = row['event_timestamp']

                # Parse datetime and make it timezone-aware
                event_timestamp = datetime.fromisoformat(event_timestamp_str)
                if event_timestamp.tzinfo is None:
                    # If naive, assume UTC
                    event_timestamp = pytz.UTC.localize(event_timestamp)

                extra_data = {}
                if row.get('bounce_type'):
                    extra_data['bounce_type'] = row['bounce_type']
                if row.get('bounce_reason'):
                    extra_data['bounce_reason'] = row['bounce_reason']
                if row.get('click_url'):
                    extra_data['url'] = row['click_url']

                if email.add_event(event_type, event_timestamp, **extra_data):
                    events_created += 1

        self.stdout.write(f"✓ Imported {events_created} events")

        # Update message stats
        self.stdout.write("Updating message statistics...")
        for message in BrevoMessage.objects.all():
            message.update_stats()

        self.stdout.write(self.style.SUCCESS("✓ Import completed!"))



# Generated manually on 2026-01-21 23:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "brevo_analytics",
            "0002_alter_brevomessage_options_brevomessage_sent_at_and_more",
        ),
    ]

    operations = [
        # Step 1: Rename model Email → BrevoEmail (preserves data)
        migrations.RenameModel(
            old_name="Email",
            new_name="BrevoEmail",
        ),

        # Step 2: Remove unique constraint from brevo_message_id
        migrations.AlterField(
            model_name="brevoemail",
            name="brevo_message_id",
            field=models.CharField(
                db_index=True,
                help_text="Brevo's message ID (shared across campaign recipients)",
                max_length=255,
            ),
        ),

        # Step 3: Add unique_together constraint on (brevo_message_id, recipient_email)
        migrations.AlterUniqueTogether(
            name="brevoemail",
            unique_together={("brevo_message_id", "recipient_email")},
        ),

        # Step 4: Add composite index for lookups
        migrations.AddIndex(
            model_name="brevoemail",
            index=models.Index(
                fields=["brevo_message_id", "recipient_email"],
                name="brevo_email_brevo_m_5657ea_idx",
            ),
        ),
    ]



import hmac
import hashlib
import json
import logging
from django.http import JsonResponse, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.conf import settings
from django.utils import timezone
from datetime import datetime
from .models import BrevoMessage, BrevoEmail

logger = logging.getLogger(__name__)


@csrf_exempt
@require_POST
def brevo_webhook(request):
    """
    Brevo webhook endpoint for real-time event processing.

    Configure in Brevo dashboard:
    URL: https://your-domain.com/brevo-analytics/webhook/
    Events: All transactional email events
    """
    # Verify webhook signature (if configured)
    config = getattr(settings, 'BREVO_ANALYTICS', {})
    webhook_secret = config.get('WEBHOOK_SECRET')

    if webhook_secret:
        signature = request.headers.get('X-Brevo-Signature', '')
        computed_signature = hmac.new(
            webhook_secret.encode(),
            request.body,
            hashlib.sha256
        ).hexdigest()

        if signature != computed_signature:
            logger.warning("Invalid webhook signature")
            return HttpResponseBadRequest('Invalid signature')

    # Parse webhook payload
    try:
        payload = json.loads(request.body)
    except json.JSONDecodeError:
        logger.error("Invalid JSON in webhook payload")
        return HttpResponseBadRequest('Invalid JSON')

    # Extract required fields
    event_type = payload.get('event')
    message_id = payload.get('message-id')
    email_address = payload.get('email')
    subject = payload.get('subject', '')
    timestamp_unix = payload.get('ts_event')

    if not all([event_type, message_id, email_address, timestamp_unix]):
        logger.error(f"Missing required fields in webhook: {payload}")
        return HttpResponseBadRequest('Missing required fields')

    # Convert timestamp
    try:
        event_datetime = datetime.fromtimestamp(timestamp_unix, tz=timezone.utc)
    except (ValueError, OSError):
        logger.error(f"Invalid timestamp: {timestamp_unix}")
        return HttpResponseBadRequest('Invalid timestamp')

    event_date = event_datetime.date()

    # Map Brevo event name to our event type
    event_mapping = {
        'request': 'sent',
        'delivered': 'delivered',
        'hard_bounce': 'bounced',
        'soft_bounce': 'bounced',
        'blocked': 'blocked',
        'spam': 'spam',
        'unsubscribe': 'unsubscribed',
        'opened': 'opened',
        'click': 'clicked',
        'deferred': 'deferred',
    }

    our_event_type = event_mapping.get(event_type, event_type)
    is_sent_event = (our_event_type == 'sent')

    # 1. Try to find existing BrevoEmail
    try:
        email = BrevoEmail.objects.select_related('message').get(
            brevo_message_id=message_id,
            recipient_email=email_address
        )
        message = email.message
        email_created = False
        logger.debug(f"Found existing email: {message_id} to {email_address}")
    except BrevoEmail.DoesNotExist:
        email = None
        email_created = True

    # 2. If email doesn't exist and this is NOT a 'sent' event, ignore it
    #    (same logic as bulk import: only emails with 'sent' event are tracked)
    if email is None and not is_sent_event:
        logger.info(
            f"Ignoring {event_type} event for unknown email {message_id} to {email_address} "
            f"(no 'sent' event received yet)"
        )
        return JsonResponse({'status': 'ignored', 'reason': 'no_sent_event'})

    # 3. If this is a 'sent' event and email doesn't exist, create it
    if email is None and is_sent_event:
        # Get or create BrevoMessage (identified by subject + sent_date from 'sent' event)
        message, message_created = BrevoMessage.objects.get_or_create(
            subject=subject,
            sent_date=event_date,
            defaults={
                'total_sent': 0,
            }
        )

        if message_created:
            logger.info(f"Created new message: {subject} - {event_date}")

        # Create BrevoEmail with sent_at from this 'sent' event
        email = BrevoEmail.objects.create(
            brevo_message_id=message_id,
            message=message,
            recipient_email=email_address,
            sent_at=event_datetime,
            current_status='sent',
            events=[]
        )
        logger.info(f"Created new email: {message_id} to {email_address}")

    # 4. Build event data with extra fields
    extra_data = {}

    # Bounce information
    if 'bounce' in event_type:
        extra_data['bounce_type'] = 'hard' if 'hard' in event_type else 'soft'
        extra_data['bounce_reason'] = payload.get('reason', '')

    # Click information
    if event_type == 'click':
        extra_data['url'] = payload.get('link', '')

    # Open information
    if event_type == 'opened':
        extra_data['ip'] = payload.get('ip', '')
        extra_data['user_agent'] = payload.get('user_agent', '')

    # Store raw payload for debugging
    extra_data['raw'] = payload

    # 5. Add event (handles duplicates internally)
    added = email.add_event(our_event_type, event_datetime, **extra_data)

    if added:
        logger.info(f"Added event {our_event_type} for email {message_id}")
    else:
        logger.debug(f"Event {our_event_type} already exists for email {message_id}")

    return JsonResponse({'status': 'ok'})



from django.contrib import admin
from django.urls import path
from django.shortcuts import render
from django.conf import settings
from django.utils.safestring import mark_safe
import json
from .models import BrevoMessage, BrevoEmail
from .i18n import get_translations, get_breadcrumb_translations


@admin.register(BrevoMessage)
class BrevoMessageAdmin(admin.ModelAdmin):
    """
    Admin that serves only the Vue.js SPA.
    Standard list/change views are completely replaced.
    """

    # Show in admin sidebar
    def has_module_permission(self, request):
        return request.user.is_staff

    def has_view_permission(self, request, obj=None):
        return request.user.is_staff

    def get_urls(self):
        """Override all URLs to serve only the SPA"""
        # Custom URLs that bypass all standard admin views
        custom_urls = [
            path('', self.admin_site.admin_view(self.spa_view), name='brevo_analytics_brevomessage_changelist'),
        ]
        return custom_urls

    def spa_view(self, request):
        """Serve Vue.js SPA"""
        # Get language code from Django settings
        language_code = getattr(settings, 'LANGUAGE_CODE', 'en').split('-')[0]

        # Get translations
        translations = get_translations(language_code)
        breadcrumb_translations = get_breadcrumb_translations(language_code)

        # Build breadcrumb
        breadcrumb = [
            {'title': breadcrumb_translations['home'], 'url': '/admin/'},
            {'title': breadcrumb_translations['brevo_analytics'], 'url': '/admin/brevo_analytics/'},
            {'title': breadcrumb_translations['message_analysis'], 'url': None},  # Current page
        ]

        return render(request, 'brevo_analytics/spa.html', {
            'title': breadcrumb_translations['message_analysis'],
            'site_title': admin.site.site_title,
            'site_header': admin.site.site_header,
            'has_permission': True,
            'breadcrumb': breadcrumb,
            'translations_json': mark_safe(json.dumps(translations)),
            'language_code': language_code,
        })

    # Disable all standard admin actions
    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(BrevoEmail)
class BrevoEmailAdmin(admin.ModelAdmin):
    """
    Admin for Blacklist Management SPA.
    Shows as separate menu item in sidebar.
    """

    # Show in admin sidebar with custom name
    def has_module_permission(self, request):
        return request.user.is_staff

    def has_view_permission(self, request, obj=None):
        return request.user.is_staff

    def get_urls(self):
        """Override all URLs to serve only the Blacklist Management SPA"""
        custom_urls = [
            path('', self.admin_site.admin_view(self.blacklist_spa_view), name='brevo_analytics_brevoemail_changelist'),
        ]
        return custom_urls

    def blacklist_spa_view(self, request):
        """Serve Blacklist Management Vue.js SPA"""
        # Get language code from Django settings
        language_code = getattr(settings, 'LANGUAGE_CODE', 'en').split('-')[0]

        # Get translations
        translations = get_translations(language_code)
        breadcrumb_translations = get_breadcrumb_translations(language_code)

        # Build breadcrumb
        breadcrumb = [
            {'title': breadcrumb_translations['home'], 'url': '/admin/'},
            {'title': breadcrumb_translations['brevo_analytics'], 'url': '/admin/brevo_analytics/'},
            {'title': breadcrumb_translations['blacklist_management'], 'url': None},  # Current page
        ]

        return render(request, 'brevo_analytics/blacklist_spa.html', {
            'title': breadcrumb_translations['blacklist_management'],
            'site_title': admin.site.site_title,
            'site_header': admin.site.site_header,
            'has_permission': True,
            'breadcrumb': breadcrumb,
            'translations_json': mark_safe(json.dumps(translations)),
            'language_code': language_code,
        })

    # Disable all standard admin actions
    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



"""
Management command to verify statistics against Brevo API.

This command compares local statistics with what's reported by Brevo API
to help identify discrepancies.

Usage:
    python manage.py verify_brevo_stats --api-key YOUR_API_KEY [--message-id ID]
"""
from django.core.management.base import BaseCommand
from django.conf import settings
from brevo_analytics.models import BrevoMessage, BrevoEmail
import requests
from datetime import datetime, timedelta


class Command(BaseCommand):
    help = 'Verify statistics against Brevo API'

    def add_arguments(self, parser):
        parser.add_argument(
            '--api-key',
            type=str,
            help='Brevo API Key (or set BREVO_API_KEY in settings)',
            default=None
        )
        parser.add_argument(
            '--message-id',
            type=int,
            help='Verify specific message ID only',
            default=None
        )
        parser.add_argument(
            '--recent',
            type=int,
            default=10,
            help='Check only N most recent messages (default: 10)'
        )

    def handle(self, *args, **options):
        api_key = options['api_key'] or settings.BREVO_ANALYTICS.get('API_KEY', '')

        if not api_key:
            self.stderr.write(self.style.ERROR(
                'Brevo API Key required. Use --api-key or set BREVO_API_KEY in settings'
            ))
            return

        headers = {
            'accept': 'application/json',
            'api-key': api_key
        }

        # Get messages to verify
        if options['message_id']:
            messages = BrevoMessage.objects.filter(id=options['message_id'])
        else:
            messages = BrevoMessage.objects.order_by('-sent_date')[:options['recent']]

        total = messages.count()
        self.stdout.write(f"Verifying {total} message(s)...\n")

        for message in messages:
            self.stdout.write(self.style.HTTP_INFO(
                f"\n{'='*80}\n{message.subject} ({message.sent_date})\n{'='*80}"
            ))

            # Local statistics
            self.stdout.write(f"\n📊 LOCAL STATISTICS:")
            self.stdout.write(f"  Total Sent:      {message.total_sent}")
            self.stdout.write(f"  Delivered:       {message.total_delivered}")
            self.stdout.write(f"  Opened:          {message.total_opened}")
            self.stdout.write(f"  Clicked:         {message.total_clicked}")
            self.stdout.write(f"  Bounced:         {message.total_bounced}")
            self.stdout.write(f"  Blocked:         {message.total_blocked}")
            self.stdout.write(f"  Delivery Rate:   {message.delivery_rate}%")
            self.stdout.write(f"  Open Rate:       {message.open_rate}%")
            self.stdout.write(f"  Click Rate:      {message.click_rate}%")

            # Count emails by status in our DB
            email_counts = {}
            for status in ['sent', 'delivered', 'opened', 'clicked', 'bounced', 'blocked', 'deferred']:
                count = message.emails.filter(current_status=status).count()
                email_counts[status] = count

            self.stdout.write(f"\n📧 EMAIL COUNTS BY STATUS:")
            for status, count in email_counts.items():
                self.stdout.write(f"  {status:12}: {count}")

            # Try to get stats from Brevo API
            try:
                # Get a sample of emails to query
                sample_emails = message.emails.order_by('sent_at')[:5]

                if not sample_emails:
                    self.stdout.write(self.style.WARNING(
                        "\n🔍 BREVO API: No emails found in local DB"
                    ))
                    continue

                self.stdout.write(f"\n🌐 Querying Brevo API for {len(sample_emails)} sample emails...")

                api_email_data = []
                for email in sample_emails:
                    # Query by email address (required parameter)
                    params = {
                        'limit': 50,
                        'email': email.recipient_email,
                        'startDate': str(message.sent_date),
                        'endDate': str(message.sent_date),
                    }

                    response = requests.get(
                        'https://api.brevo.com/v3/smtp/emails',
                        headers=headers,
                        params=params,
                        timeout=10
                    )

                    if response.status_code == 200:
                        data = response.json()
                        emails_list = data.get('transactionalEmails', [])
                        # Filter by subject
                        matching = [e for e in emails_list if e.get('subject') == message.subject]
                        api_email_data.extend(matching)

                # Remove duplicates by messageId
                seen_ids = set()
                unique_api_emails = []
                for email_data in api_email_data:
                    msg_id = email_data.get('messageId')
                    if msg_id and msg_id not in seen_ids:
                        seen_ids.add(msg_id)
                        unique_api_emails.append(email_data)

                if unique_api_emails:
                    self.stdout.write(f"\n🔍 BREVO API RESULTS:")
                    self.stdout.write(f"  Sample emails queried: {len(sample_emails)}")
                    self.stdout.write(f"  Matching emails found:  {len(unique_api_emails)}")

                    # Skip detailed event counting for now
                    self.stdout.write(f"\n  (Detailed event comparison skipped - API returns limited data)")
                else:
                    self.stdout.write(self.style.WARNING(
                        "\n🔍 BREVO API: No matching emails found"
                    ))
                    continue

                # Now try to get aggregate stats instead
                response = None  # Skip the old response handling

                if response.status_code == 200:
                    data = response.json()
                    emails = data.get('transactionalEmails', [])

                    # Filter by subject
                    matching_emails = [e for e in emails if e.get('subject') == message.subject]

                    self.stdout.write(f"\n🔍 BREVO API RESULTS:")
                    self.stdout.write(f"  Total emails returned: {len(emails)}")
                    self.stdout.write(f"  Matching subject:      {len(matching_emails)}")

                    if matching_emails:
                        # Count by event
                        api_counts = {
                            'sent': 0,
                            'delivered': 0,
                            'opened': 0,
                            'clicked': 0,
                            'bounced': 0,
                            'blocked': 0,
                        }

                        for email in matching_emails:
                            events = email.get('event', [])
                            if isinstance(events, list):
                                if 'click' in events:
                                    api_counts['clicked'] += 1
                                elif 'opened' in events:
                                    api_counts['opened'] += 1
                                elif 'delivered' in events:
                                    api_counts['delivered'] += 1
                                elif 'hard_bounce' in events or 'soft_bounce' in events:
                                    api_counts['bounced'] += 1
                                elif 'blocked' in events:
                                    api_counts['blocked'] += 1
                                else:
                                    api_counts['sent'] += 1

                        self.stdout.write(f"\n  API Event Counts:")
                        for status, count in api_counts.items():
                            local_count = email_counts.get(status, 0)
                            diff = count - local_count
                            diff_str = f"({diff:+d})" if diff != 0 else "✓"
                            self.stdout.write(f"    {status:12}: {count:4} vs {local_count:4} {diff_str}")

                    else:
                        self.stdout.write(self.style.WARNING(
                            "  No matching emails found in API response"
                        ))

                else:
                    try:
                        error_detail = response.json()
                        self.stderr.write(self.style.ERROR(
                            f"  API error {response.status_code}: {error_detail}"
                        ))
                    except:
                        self.stderr.write(self.style.ERROR(
                            f"  API error {response.status_code}: {response.text[:500]}"
                        ))

            except Exception as e:
                self.stderr.write(self.style.ERROR(
                    f"  Error querying API: {str(e)}"
                ))
                import traceback
                self.stderr.write(traceback.format_exc())

        self.stdout.write(f"\n{'='*80}\n")
        self.stdout.write(self.style.SUCCESS("✓ Verification complete"))



from django.urls import path
from . import api_views, webhooks

app_name = 'brevo_analytics'

urlpatterns = [
    # API endpoints
    path('api/dashboard/', api_views.dashboard_api, name='api_dashboard'),
    path('api/messages/', api_views.messages_list_api, name='api_messages'),
    path('api/messages/<int:message_id>/emails/', api_views.message_emails_api, name='api_message_emails'),
    path('api/emails/bounced/', api_views.emails_bounced_api, name='api_emails_bounced'),
    path('api/emails/blocked/', api_views.emails_blocked_api, name='api_emails_blocked'),

    # Blacklist management
    path('api/blacklist/', api_views.list_blacklist_api, name='api_list_blacklist'),
    path('api/blacklist/enrich/', api_views.enrich_blocked_emails_api, name='api_enrich_blacklist'),
    path('api/blacklist/<str:email_address>/', api_views.check_blacklist_status_api, name='api_check_blacklist'),
    path('api/blacklist/<str:email_address>/remove/', api_views.remove_from_blacklist_api, name='api_remove_blacklist'),
    path('api/bounce-details/<str:email_address>/', api_views.get_bounce_details_api, name='api_bounce_details'),

    path('api/emails/<uuid:email_id>/', api_views.email_detail_api, name='api_email_detail'),

    # Webhook
    path('webhook/', webhooks.brevo_webhook, name='webhook'),
]



"""Custom template filters for Brevo Analytics."""
from django import template
from django.utils import timezone
from datetime import datetime, timedelta

register = template.Library()


@register.filter
def format_seconds(seconds):
    """
    Convert seconds to human-readable duration.

    Args:
        seconds: Number of seconds (can be int, float, or string)

    Returns:
        Formatted string like "2h 30m" or "45s"
    """
    try:
        seconds = float(seconds)
    except (TypeError, ValueError):
        return "N/A"

    if seconds < 0:
        return "N/A"

    if seconds < 60:
        return f"{int(seconds)}s"

    minutes = int(seconds // 60)
    if minutes < 60:
        return f"{minutes}m"

    hours = int(minutes // 60)
    remaining_minutes = minutes % 60

    if remaining_minutes == 0:
        return f"{hours}h"

    return f"{hours}h {remaining_minutes}m"


@register.filter
def event_icon(event_type):
    """
    Return an emoji icon for the given event type.

    Args:
        event_type: String representing the event type

    Returns:
        Emoji string
    """
    icons = {
        'delivered': '✅',
        'opened': '👁️',
        'clicked': '🖱️',
        'soft_bounce': '⚠️',
        'hard_bounce': '❌',
        'spam': '🚫',
        'blocked': '🛑',
        'unsubscribed': '📭',
        'error': '⚠️',
        'deferred': '⏸️',
        'complaint': '🚨',
    }

    if not event_type:
        return '📧'

    return icons.get(str(event_type).lower(), '📧')


@register.filter
def status_color(status):
    """
    Return a CSS class for styling based on status.

    Args:
        status: String representing the status

    Returns:
        CSS class string
    """
    color_map = {
        'delivered': 'text-green-600 bg-green-100',
        'opened': 'text-blue-600 bg-blue-100',
        'clicked': 'text-purple-600 bg-purple-100',
        'soft_bounce': 'text-yellow-600 bg-yellow-100',
        'hard_bounce': 'text-red-600 bg-red-100',
        'spam': 'text-red-700 bg-red-200',
        'blocked': 'text-red-800 bg-red-200',
        'unsubscribed': 'text-gray-600 bg-gray-100',
        'error': 'text-orange-600 bg-orange-100',
        'deferred': 'text-yellow-500 bg-yellow-50',
        'complaint': 'text-red-900 bg-red-300',
    }

    if not status:
        return 'text-gray-500 bg-gray-50'

    return color_map.get(str(status).lower(), 'text-gray-500 bg-gray-50')


@register.filter
def time_since_sent(sent_time):
    """
    Calculate time elapsed since email was sent.

    Args:
        sent_time: datetime object or ISO format string

    Returns:
        Human-readable string like "2 hours ago" or "3 days ago"
    """
    if not sent_time:
        return "N/A"

    # Convert string to datetime if needed
    if isinstance(sent_time, str):
        try:
            sent_time = datetime.fromisoformat(sent_time.replace('Z', '+00:00'))
        except (ValueError, AttributeError):
            return "N/A"

    # Make sure we have a timezone-aware datetime
    if sent_time.tzinfo is None:
        sent_time = timezone.make_aware(sent_time)

    now = timezone.now()
    delta = now - sent_time

    # Handle future dates
    if delta.total_seconds() < 0:
        return "just now"

    seconds = int(delta.total_seconds())

    if seconds < 60:
        return f"{seconds}s ago"

    minutes = seconds // 60
    if minutes < 60:
        return f"{minutes}m ago"

    hours = minutes // 60
    if hours < 24:
        return f"{hours}h ago"

    days = hours // 24
    if days < 7:
        return f"{days}d ago"

    weeks = days // 7
    if weeks < 4:
        return f"{weeks}w ago"

    months = days // 30
    if months < 12:
        return f"{months}mo ago"

    years = days // 365
    return f"{years}y ago"


@register.filter
def replace(value, arg):
    """
    Replace occurrences of a substring in a string.

    Usage: {{ "hello_world"|replace:"_: " }}

    Args:
        value: The string to search in
        arg: String in format "search:replace" (separated by colon)

    Returns:
        String with replacements made
    """
    if not value or not arg:
        return value

    try:
        search, replace_with = arg.split(':', 1)
        return str(value).replace(search, replace_with)
    except (ValueError, AttributeError):
        return value


@register.filter
def format_event_type(event_type):
    """
    Format event type for display (e.g., 'hard_bounce' -> 'Hard Bounce').

    Args:
        event_type: String representing the event type

    Returns:
        Formatted string
    """
    if not event_type:
        return ""

    # Replace underscores with spaces and capitalize each word
    return str(event_type).replace('_', ' ').title()


@register.filter
def smart_timestamp(timestamp):
    """
    Format timestamp with relative time and absolute time.

    - If today (after midnight): "6h ago - 05:55"
    - If before today: "32h ago - 20/01 14:43"

    Args:
        timestamp: datetime object or ISO format string

    Returns:
        Formatted string
    """
    if not timestamp:
        return "N/A"

    # Convert string to datetime if needed
    if isinstance(timestamp, str):
        try:
            timestamp = datetime.fromisoformat(timestamp.replace('Z', '+00:00'))
        except (ValueError, AttributeError):
            return "N/A"

    # Make sure we have a timezone-aware datetime
    if timestamp.tzinfo is None:
        timestamp = timezone.make_aware(timestamp)

    now = timezone.now()
    delta = now - timestamp

    # Handle future dates
    if delta.total_seconds() < 0:
        return "just now"

    # Calculate relative time
    seconds = int(delta.total_seconds())

    if seconds < 60:
        relative = f"{seconds}s ago"
    elif seconds < 3600:
        relative = f"{seconds // 60}m ago"
    else:
        relative = f"{seconds // 3600}h ago"

    # Get today's midnight
    today_midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)

    # Format absolute time based on whether it's today or before
    if timestamp >= today_midnight:
        # Today: show only HH:MM
        absolute = timestamp.strftime("%H:%M")
    else:
        # Before today: show dd/MM HH:MM
        absolute = timestamp.strftime("%d/%m %H:%M")

    return f"{relative} - {absolute}"



from rest_framework import serializers
from .models import BrevoMessage, BrevoEmail


class BrevoMessageSerializer(serializers.ModelSerializer):
    """Serializer per lista messaggi"""
    class Meta:
        model = BrevoMessage
        fields = [
            'id', 'subject', 'sent_date',
            'total_sent', 'total_delivered', 'total_opened',
            'total_clicked', 'total_bounced', 'total_blocked',
            'delivery_rate', 'open_rate', 'click_rate',
            'updated_at'
        ]


class BrevoEmailListSerializer(serializers.ModelSerializer):
    """Serializer per lista email (senza eventi)"""
    class Meta:
        model = BrevoEmail
        fields = [
            'id', 'recipient_email', 'current_status', 'sent_at'
        ]


class BrevoEmailDetailSerializer(serializers.ModelSerializer):
    """Serializer per dettaglio email (con eventi e messaggio)"""
    message = serializers.SerializerMethodField()

    class Meta:
        model = BrevoEmail
        fields = [
            'id', 'recipient_email', 'current_status',
            'sent_at', 'events', 'message', 'blacklist_info'
        ]

    def get_message(self, obj):
        return {
            'id': obj.message.id,
            'subject': obj.message.subject,
            'sent_date': obj.message.sent_date.isoformat()
        }


class MessageBrevoEmailsSerializer(serializers.Serializer):
    """Serializer per risposta /api/messages/:id/emails/"""
    message = BrevoMessageSerializer()
    emails = BrevoEmailListSerializer(many=True)


class GlobalBrevoEmailsSerializer(serializers.ModelSerializer):
    """Serializer per email globali bounced/blocked (con info messaggio)"""
    message = serializers.SerializerMethodField()

    class Meta:
        model = BrevoEmail
        fields = [
            'id', 'recipient_email', 'current_status',
            'sent_at', 'message'
        ]

    def get_message(self, obj):
        return {
            'subject': obj.message.subject,
            'sent_date': obj.message.sent_date.isoformat()
        }
